<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\Logger;

use Dedi\SyliusMigrationPlugin\Definition\TableMappingDefinition;
use Dedi\SyliusMigrationPlugin\Exception\MediaDownloadException;
use Doctrine\DBAL\Exception\ConstraintViolationException;
use Exception;
use InvalidArgumentException;
use PDOException;
use Psr\Log\LoggerInterface;
use Psr\Log\LogLevel;

class MigrationLogger implements MigrationLoggerInterface
{
    private readonly LoggerInterface $logger;

    public function __construct(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    public function debug(TableMappingDefinition $tableMappingDefinition, string $message, array $data = []): void
    {
        $contextString = $this->getContextString($tableMappingDefinition, $data[$tableMappingDefinition->getSourcePrimaryKey()] ?? '');
        $contextData = $this->getContextData($tableMappingDefinition, $data);

        $this->log(LogLevel::DEBUG, sprintf('%s %s', $contextString, $message), $contextData);
    }

    public function notice(TableMappingDefinition $tableMappingDefinition, string $message, array $data = []): void
    {
        $contextString = $this->getContextString($tableMappingDefinition, $data[$tableMappingDefinition->getSourcePrimaryKey()] ?? '');
        $contextData = $this->getContextData($tableMappingDefinition, $data);

        $this->log(LogLevel::NOTICE, sprintf('%s %s', $contextString, $message), $contextData);
    }

    public function info(TableMappingDefinition $tableMappingDefinition, string $message, array $data = []): void
    {
        $contextString = $this->getContextString($tableMappingDefinition, $data[$tableMappingDefinition->getSourcePrimaryKey()] ?? '');
        $contextData = $this->getContextData($tableMappingDefinition, $data);

        $this->log(LogLevel::INFO, sprintf('%s %s', $contextString, $message), $contextData);
    }

    public function error(TableMappingDefinition $tableMappingDefinition, string $message, array $data = []): void
    {
        $contextString = $this->getContextString($tableMappingDefinition, $data[$tableMappingDefinition->getSourcePrimaryKey()] ?? '');
        $contextData = $this->getContextData($tableMappingDefinition, $data);

        $this->log(LogLevel::ERROR, sprintf("'%s %s'", $contextString, $message), $contextData);
    }

    public function warning(TableMappingDefinition $tableMappingDefinition, string $message, array $data = []): void
    {
        $contextString = $this->getContextString($tableMappingDefinition, $data[$tableMappingDefinition->getSourcePrimaryKey()] ?? '');
        $contextData = $this->getContextData($tableMappingDefinition, $data);

        $this->log(LogLevel::WARNING, sprintf('%s %s', $contextString, $message), $contextData);
    }

    public function critical(TableMappingDefinition $tableMappingDefinition, string $message, array $data = []): void
    {
        $contextString = $this->getContextString($tableMappingDefinition, $data[$tableMappingDefinition->getSourcePrimaryKey()] ?? '');
        $contextData = $this->getContextData($tableMappingDefinition, $data);

        $this->log(LogLevel::CRITICAL, sprintf("'%s %s'", $contextString, $message), $contextData);
    }

    public function log($level, string $message, array $context = []): void
    {
        switch ($level) {
            case LogLevel::DEBUG:
                $this->logger->debug($message, $context);

                break;
            case LogLevel::NOTICE:
                $this->logger->notice($message, $context);

                break;
            case LogLevel::INFO:
                $this->logger->info($message, $context);

                break;
            case LogLevel::ERROR:
                $this->logger->error($message, $context);

                break;
            case LogLevel::WARNING:
                $this->logger->warning($message, $context);

                break;
            case LogLevel::CRITICAL:
                $this->logger->critical($message, $context);

                break;
            default:
                throw new InvalidArgumentException(sprintf('Unrecognized level %s', $level));
        }
    }

    public function handleException(Exception $exception, TableMappingDefinition $tableMappingDefinition, $data = [])
    {
        $previous = $exception->getPrevious();
        $message = $previous instanceof PDOException ? $previous->getMessage() : $exception->getMessage();

        switch (true) {
            case $exception instanceof ConstraintViolationException:
            case $exception instanceof MediaDownloadException:
                $this->error($tableMappingDefinition, $exception->getMessage(), array_merge($data, ['message' => $message]));

                break;
            default:
                $this->critical($tableMappingDefinition, $exception->getMessage(), array_merge($data, ['message' => $message]));

                throw $exception;
        }
    }

    private function getContextString(TableMappingDefinition $tableMappingDefinition, $data = null): string
    {
        if (null === $data || '' === $data || (is_array($data) && 0 === count($data))) {
            return '>';
        }

        $sourceString = trim(sprintf(
            '%s.%s %s',
            '' !== $tableMappingDefinition->getSourceTableName() ? $tableMappingDefinition->getSourceTableName() : 'sql',
            $tableMappingDefinition->getSourcePrimaryKey(),
            $data,
        ));

        $contextString = trim(sprintf(
            '[%s -> %s]',
            $sourceString,
            $tableMappingDefinition->getTargetTableName(),
        ));

        return $contextString;
    }

    private function getContextData(TableMappingDefinition $tableMappingDefinition, array $data)
    {
        if (0 === count($data)) {
            return [];
        }

        $return = [
            'id' => $tableMappingDefinition->getSourcePrimaryKey(),
            'primary_key' => $data[$tableMappingDefinition->getSourcePrimaryKey()] ?? '',
            'source_table' => '' !== $tableMappingDefinition->getSourceTableName() ? $tableMappingDefinition->getSourceTableName() : 'SQL',
            'target_table' => $tableMappingDefinition->getTargetTableName(),
        ];

        foreach ($tableMappingDefinition->getLoggingColumns() as $loggingColumn) {
            $return[$loggingColumn] = $data[$loggingColumn] ?? '';
        }

        $return['message'] = $data['message'] ?? '';

        return $return;
    }
}
